/*** * **** * ***** ********* ** ****** ***** *** ***** ********
**
** Copyright (c) 2005-2009, SkeTo Project
** All rights reserved.
**
** This file is part of the SkeTo Library. This library is free
** software; you can redistribute it and/or modify it under the
** terms of the MIT License (see COPYING in the package).
** 
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
**
*** * **** * ***** ********* ** ****** ***** *** ***** ********/

/**
 * @file list_skeletons_noopt.h
 * @brief An implmentation of list skeletons without fusion optimization.
 *        These skeletons are for debugging.
 */

#ifndef __LIST_SKELETONS_NOOPT_H__
#define __LIST_SKELETONS_NOOPT_H__

namespace sketo {
  namespace list_skeletons {
    /**
     * generate :: int -> (int -> a) -> [a]
     */
    template <typename F>
    dist_list<typename F::result_type>
    generate(int global_size,
	     const F& f);

    /**
     * map :: (a -> b) -> [a] -> [b]
     */
    template <typename F, typename A>
    dist_list<typename F::result_type>
    map(F f,
	const sketo::dist_list<A>& as);

    /**
     * map_with_index :: (int -> a -> b) -> [a] -> [b]
     */
    template <typename F, typename A>
    dist_list<typename F::result_type>
    map_with_index(const F& f,
		   const dist_list<A>& as);

    /**
     * zip :: [a] -> [b] -> [(a, b)]
     */
    template <typename A, typename B>
    dist_list<std::pair<A, B> >
    zip(const dist_list<A>& as,
	const dist_list<B>& bs);

    /**
     * zipwith :: (a -> b -> c) -> [a] -> [b] -> [c]
     */
    template <typename F, typename A, typename B>
    dist_list<typename F::result_type>
    zipwith(const F& f,
	    const dist_list<A>& as,
	    const dist_list<B>& bs);

    /**
     * reduce :: (a -> a -> a) -> [a] -> [a]
     */
    template <typename OPLUS>
    typename OPLUS::result_type
    reduce(const OPLUS &oplus,
	   const dist_list<typename OPLUS::first_argument_type>& as);

    /**
     * scan :: (a -> a -> a) -> (a, [a]) -> ([a], a)
     */
    template <typename OPLUS>
    dist_list<typename OPLUS::result_type>
    scan(const OPLUS &oplus,
	 typename OPLUS::first_argument_type e,
	 const dist_list<typename OPLUS::second_argument_type>& as,
	 typename OPLUS::result_type *last_val = NULL);

    /**
     * scanr :: (a -> a -> a) -> ([a], a) -> (a, [a])
     */
    template <typename OPLUS>
    dist_list<typename OPLUS::result_type>
    scanr(const OPLUS &oplus,
	  typename OPLUS::second_argument_type e,
	  const dist_list<typename OPLUS::first_argument_type>& as,
	  typename OPLUS::result_type *first_val = NULL);

    /**
     * postscan :: (a -> a -> a) -> [a] -> [a]
     */
    template <typename OPLUS>
    dist_list<typename OPLUS::result_type>
    postscan(const OPLUS &oplus,
	     const dist_list<typename OPLUS::first_argument_type>& as);

    /**
     * postscanr :: (a -> a -> a) -> [a] -> [a]
     */
    template <typename OPLUS>
    dist_list<typename OPLUS::result_type>
    postscanr(const OPLUS &oplus,
	      const dist_list<typename OPLUS::second_argument_type>& as);

    /**
     * gscanl :: (b -> a -> b) -> b -> (a -> c) -> (c -> c -> c) -> (b -> c -> b) -> [a] -> [b]
     */
    template <typename F, typename G, typename OMINUS, typename OPLUS>
    dist_list<typename F::result_type>
    gscanl(const F &f,
	   const typename F::first_argument_type &c,
	   const G &g,
	   const OMINUS &ominus,
	   const OPLUS &oplus,
	   const dist_list<typename F::second_argument_type>& as);

    /**
     * gscanr :: (b -> a -> b) -> b -> (a -> c) -> (c -> c -> c) -> (b -> c -> b) -> [a] -> [b]
     */
    template <typename F, typename G, typename OMINUS, typename OPLUS>
    dist_list<typename F::result_type>
    gscanr(const F &f,
	   const typename F::first_argument_type &c,
	   const G &g,
	   const OMINUS &ominus,
	   const OPLUS &oplus,
	   const dist_list<typename F::second_argument_type>& as);

    /**
     * shiftl :: ([a], a) -> (a, [a])
     */
    template <typename A>
    dist_list<A>
    shiftl(const A &rightmost, 
	   const dist_list<A> &as,
	   A *leftmost = NULL);

    /**
     * shiftr :: (a, [a]) -> ([a], a)
     */
    template <typename A>
    dist_list<A>
    shiftr(const A &leftmost,
	   const dist_list<A> &as,
	   A *rightmost = NULL);

    /**
     * @todo write comments
     */
    template <typename A>
    void
    print(const dist_list<A>& as,
	  std::ostream &stream = std::cout);
    
  }
}

#include "list_skeletons_noopt.tpp"

#endif //__LIST_SKELETONS_WITHOUT_FUSION_H__
