/*** * **** * ***** ********* ** ****** ***** *** ***** ********
**
** Copyright (c) 2005-2009, SkeTo Project
** All rights reserved.
**
** This file is part of the SkeTo Library. This library is free
** software; you can redistribute it and/or modify it under the
** terms of the MIT License (see COPYING in the package).
** 
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
**
*** * **** * ***** ********* ** ****** ***** *** ***** ********/

/**
 * @mainpage
 *
 * SkeTo (Skeletons in Tokyo) library provides a set of parallel
 * skeletons for lists, matrices, and trees.  A parallel skeleton is a
 * computation pattern which is often used in parallel programming.
 * In our SkeTo library, a parallel skeleton provides a sequential
 * interface for manipulating (distributed) data strucuters and at the
 * same time it is implemented in parallel efficiently.  These
 * parallel skeletons are a more straightforward way to compose
 * parallel programs.
 *
 * Copyright (c) 2005-2009, SkeTo Project.  All rights reserved.
 */
 
/**
 * @file sketo.h
 * @brief The skeleton common file.
 * @author Yoshiki AKASHI, Kiminori MATSUZAKI, Kento EMOTO
 */

#ifndef __SKETO_H__
#define __SKETO_H__

#include <mpi.h>

#include <string>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <cstdarg>
#include <vector>
#include <utility>

namespace sketo {
  extern int rank;  // MPI rank
  extern int procs; // Number of processes

  int main(int, char**); // User should implement this function.

  void init(int &argc, char **&argv);
  void finalize();

  inline bool is_master() { return (rank == 0); }
}

#include "exceptions.h"
#include "functions.h"
#include "sketo_io.h"
#include "sketo_time.h"

#endif // __SKETO_H__
