/*** * **** * ***** ********* ** ****** ***** *** ***** ********
**
** Copyright (c) 2005-2009, SkeTo Project
** All rights reserved.
**
** This file is part of the SkeTo Library. This library is free
** software; you can redistribute it and/or modify it under the
** terms of the MIT License (see COPYING in the package).
** 
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
**
*** * **** * ***** ********* ** ****** ***** *** ***** ********/

/**
 * @file line_of_sight.cpp
 * @brief This program search lines which you can see.
 * @author Yoshiki AKASHI (yoshiki@ipl.cs.uec.ac.jp)
 * @author: Kiminori Matsuzaki (kmatsu@ipl.t.u-tokyo.ac.jp; converted to new sketo implementation)
 */

#include <cstdlib>
#include <cfloat>
#include <ctime>
#include <sketo/list_skeletons.h>

struct gen_t : public sketo::functions::base <double (int)> {
  double operator()(int index) const {
    return static_cast<double>((rand() % 10) * (index + 1));
  }
} gen;

struct divide_by_index_t : public sketo::functions::base <double (int, double)> {
  double operator()(int index, double x) const {
    return x / (static_cast<double>(index) + 1.0);
  }
} divide_by_index;

struct greater_filter_t : public sketo::functions::base <int (double, double)> {
  int operator()(double x, double y) const {
    return (x > y) ? 1 : 0;
  }
} greater_filter;

int sketo::main(int argc, char **argv)
{
  using namespace sketo;
  using namespace sketo::list_skeletons;
  
  if (argc < 3) {
    sketo::cout << "usage: " << argv[0] << " size loop" << std::endl;
    return 1;
  }

  const int size = std::atoi(argv[1]);
  const int loop = std::atoi(argv[2]);

  std::srand(std::time(NULL) + sketo::rank);
//   skeleton::time::initialize( ); // for time keeping

  for (int i = 0; i < loop; ++i) {
    // generate data
    dist_list<double> as = generate(size, gen);
    
    // calculate angle
    as = map_with_index(divide_by_index, as);
    
    // calculate max angle before oneself
    dist_list<double> bs = scan(functions::max<double>(), -1, as);

    // if the angle equal max angle, you can see it.
    dist_list<int> results = zipwith(greater_filter, as, bs);

#ifdef DEBUG_OUTPUT
    print(as);
    print(bs);
    print(results);
#endif
  }
//   skeleton::time::set_time("calculate"); // for time keeping
//   skeleton::time::print(); // show the result of time keeping

  return 0;
}
