/*** * **** * ***** ********* ** ****** ***** *** ***** ********
**
** Copyright (c) 2005-2009, SkeTo Project
** All rights reserved.
**
** This file is part of the SkeTo Library. This library is free
** software; you can redistribute it and/or modify it under the
** terms of the MIT License (see COPYING in the package).
** 
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
**
*** * **** * ***** ********* ** ****** ***** *** ***** ********/
 
/**
 * @file sketo.cpp
 * @brief The skeleton common file.
 * @author Kiminori MATSUZAKI (kmatus@ipl.t.u-tokyo.ac.jp)
 * @author Yoshiki AKASHI (yoshiki@ipl.cs.uec.ac.jp)
 */

#include "../include/sketo/sketo.h"

namespace sketo {
  int rank;  // MPI rank
  int procs; // Number of processors
  single_ostream cout;

  void init(int &argc, char **&argv) {
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    MPI_Comm_size(MPI_COMM_WORLD, &procs);
  }

  void finalize() {
    MPI_Finalize();
  }

  void mprintf(const char* format, ...) {
    if (!is_master()) return;
    va_list args;
    va_start(args, format);
    vprintf(format, args);
    va_end(args);
  }

};

int main(int argc, char **argv) {
  sketo::init(argc, argv);

  int retval = 0;
  try {
    retval = sketo::main(argc, argv);
//   } catch (sketo::bad_size e) {
//     std::cerr << "bad_size exception: (" << e.size_a << ", " << e.size_b << ")" << std::endl;
//     retval = -1;
//   } catch (sketo::error e) {
//     std::cerr << "Error: " << e.msg << std::endl;
//     retval = e._errno;
  } catch (...) {
    std::cerr << "Something Wrong" << std::endl;
  }

  sketo::finalize();
  return retval;
}

#if 0 /* old source */
namespace sketo {
  namespace time {
    std::vector<std::pair<double, std::string> > time_table;
    void initialize() {
      if (is_master()) time_table.clear();
      MPI_Barrier(MPI_COMM_WORLD);
      if (is_master()) time_table.push_back(std::pair<double, std::string>(MPI_Wtime(), std::string("")));
    }

    void set_time(const char* name, bool barrier) {
      if (barrier) MPI_Barrier(MPI_COMM_WORLD);
      if (is_master()) time_table.push_back(std::pair<double, std::string>(MPI_Wtime(),std::string(name)));
    }

    void print(FILE *fp) {
      if (!is_master()) return;
      fprintf(fp, "%10s %10s %s\n", "total", "difference", "comment");
      for (unsigned int i = 1; i < time_table.size(); i++) {
        fprintf(fp, "%10.6f %10.6f %s\n",
                time_table[i].first - time_table[0].first,
                time_table[i].first - time_table[i-1].first,
                time_table[i].second.c_str());
      }
    }
  }
}
#endif
